@@ -1,5 +1,6 @@
 import logging
 import os
+import shutil
 import sqlite3
 from datetime import datetime
 from functools import wraps
@@ -9,6 +10,19 @@ from typing import Any
 from tabulate import tabulate
 
 
+def _connect(func):
+    """Decorator to open db connection if it isn't already open."""
+
+    @wraps(func)
+    def inner(self, *args, **kwargs):
+        if not self.connection_open:
+            self.open()
+        results = func(self, *args, **kwargs)
+        return results
+
+    return inner
+
+
 class DataBased:
     """Sqli wrapper so queries don't need to be written except table definitions.
 
@@ -32,6 +46,7 @@ class DataBased:
         for the logger object."""
         self.dbpath = Path(dbpath)
         self.dbname = Path(dbpath).name
+        self.dbpath.parent.mkdir(parents=True, exist_ok=True)
         self._logger_init(
             encoding=logger_encoding, message_format=logger_message_format
         )
@@ -51,7 +66,7 @@ class DataBased:
         manager_template = Path(__file__).parent / "dbmanager.py"
         manager_path = self.dbpath.parent / "dbmanager.py"
         if not manager_path.exists():
-            manager_path.write_text(manager_template.read_text())
+            shutil.copyfile(manager_template, manager_path)
 
     def open(self):
         """Open connection to db."""
@@ -74,19 +89,6 @@ class DataBased:
             self.connection.close()
             self.connection_open = False
 
-    def _connect(func):
-        """Decorator to open db connection if it isn't already open."""
-
-        @wraps(func)
-        def inner(*args, **kwargs):
-            self = args[0]
-            if not self.connection_open:
-                self.open()
-            results = func(*args, **kwargs)
-            return results
-
-        return inner
-
     def _logger_init(
         self,
         message_format: str = "{levelname}|-|{asctime}|-|{message}",
@@ -155,14 +157,21 @@ class DataBased:
         return f"({conditions})"
 
     @_connect
-    def create_tables(self, table_statements: list[str] = []):
+    def query(self, query_) -> list[Any]:
+        """Execute an arbitrary query and
+        return the results."""
+        self.cursor.execute(query_)
+        return self.cursor.fetchall()
+
+    @_connect
+    def create_tables(self, table_querys: list[str] = []):
         """Create tables if they don't exist.
 
-        :param table_statements: Each statement should be
+        :param table_querys: Each query should be
         in the form 'tableName(columnDefinitions)'"""
-        if len(table_statements) > 0:
+        if len(table_querys) > 0:
             table_names = self.get_table_names()
-            for table in table_statements:
+            for table in table_querys:
                 if table.split("(")[0].strip() not in table_names:
                     self.cursor.execute(f"create table {table}")
                     self.logger.info(f'{table.split("(")[0]} table created.')
@@ -177,8 +186,8 @@ class DataBased:
         proper Sqlite3 sytax.
         i.e. "columnName text unique" or "columnName int primary key" etc."""
         if table not in self.get_table_names():
-            statement = f"create table {table}({', '.join(column_defs)})"
-            self.cursor.execute(statement)
+            query = f"create table {table}({', '.join(column_defs)})"
+            self.cursor.execute(query)
             self.logger.info(f"'{table}' table created.")
 
     @_connect
@@ -213,14 +222,14 @@ class DataBased:
         in match_criteria will be matched as a substring. Has no effect if
         match_criteria is None.
         """
-        statement = f"select count(_rowid_) from {table}"
+        query = f"select count(_rowid_) from {table}"
         try:
             if match_criteria:
                 self.cursor.execute(
-                    f"{statement} where {self._get_conditions(match_criteria, exact_match)}"
+                    f"{query} where {self._get_conditions(match_criteria, exact_match)}"
                 )
             else:
-                self.cursor.execute(f"{statement}")
+                self.cursor.execute(f"{query}")
             return self.cursor.fetchone()[0]
         except:
             return 0
@@ -266,7 +275,9 @@ class DataBased:
         sort_by_column: str = None,
         columns_to_return: list[str] = None,
         values_only: bool = False,
-    ) -> tuple[dict] | tuple[tuple]:
+        order_by: str = None,
+        limit: str | int = None,
+    ) -> list[dict] | list[tuple]:
         """Returns rows from table as a list of dictionaries
         where the key-value pairs of the dictionaries are
         column name: row value.
@@ -279,35 +290,43 @@ class DataBased:
         will be matched as a substring.
 
         :param sort_by_column: A column name to sort the results by.
+        This will sort results in Python after retrieving them from the db.
+        Use the 'order_by' param to use SQLite engine for ordering.
 
         :param columns_to_return: Optional list of column names.
-        If provided, the dictionaries returned by get_rows() will
+        If provided, the elements returned by get_rows() will
         only contain the provided columns. Otherwise every column
         in the row is returned.
 
-        :param values_only: Return the results as a tuple of tuples
-        instead of a tuple of dictionaries that have column names as keys.
+        :param values_only: Return the results as a list of tuples
+        instead of a list of dictionaries that have column names as keys.
         The results will still be sorted according to sort_by_column if
         one is provided.
+
+        :param order_by: If given, a 'order by {order_by}' clause
+        will be added to the select query.
+
+        :param limit: If given, a 'limit {limit}' clause will be
+        added to the select query.
         """
         if type(columns_to_return) is str:
             columns_to_return = [columns_to_return]
-        statement = f"select * from {table}"
+        query = f"select * from {table}"
         matches = []
-        if not match_criteria:
-            self.cursor.execute(statement)
-        else:
-            self.cursor.execute(
-                f"{statement} where {self._get_conditions(match_criteria, exact_match)}"
-            )
+        if match_criteria:
+            query += f" where {self._get_conditions(match_criteria, exact_match)}"
+        if order_by:
+            query += f" order by {order_by}"
+        if limit:
+            query += f" limit {limit}"
+        query += ";"
+        self.cursor.execute(query)
         matches = self.cursor.fetchall()
-        results = tuple(
-            self._get_dict(table, match, columns_to_return) for match in matches
-        )
+        results = [self._get_dict(table, match, columns_to_return) for match in matches]
         if sort_by_column:
-            results = tuple(sorted(results, key=lambda x: x[sort_by_column]))
+            results = sorted(results, key=lambda x: x[sort_by_column])
         if values_only:
-            return tuple(tuple(row.values()) for row in results)
+            return [tuple(row.values()) for row in results]
         else:
             return results
 
@@ -340,7 +359,7 @@ class DataBased:
                     if row not in results
                 ]
             )
-        return tuple(results)
+        return results
 
     @_connect
     def delete(
@@ -389,7 +408,7 @@ class DataBased:
         If None, every row will be updated.
 
         Returns True if successful, False if not."""
-        statement = f"update {table} set {column_to_update} = ?"
+        query = f"update {table} set {column_to_update} = ?"
         if match_criteria:
             if self.count(table, match_criteria) == 0:
                 self.logger.info(
@@ -397,12 +416,12 @@ class DataBased:
                 )
                 return False
             conditions = self._get_conditions(match_criteria)
-            statement += f" where {conditions}"
+            query += f" where {conditions}"
         else:
             conditions = None
         try:
             self.cursor.execute(
-                statement,
+                query,
                 (new_value,),
             )
             self.logger.info(
@@ -457,6 +476,22 @@ class DataBased:
         except Exception as e:
             self.logger.error(f'Failed to add column "{column}" to "{table}" table.')
 
+    @staticmethod
+    def data_to_string(
+        data: list[dict], sort_key: str = None, wrap_to_terminal: bool = True
+    ) -> str:
+        """Uses tabulate to produce pretty string output
+        from a list of dictionaries.
+
+        :param data: Assumes all dictionaries in list have the same set of keys.
+
+        :param sort_key: Optional dictionary key to sort data with.
+
+        :param wrap_to_terminal: If True, the table width will be wrapped
+        to fit within the current terminal window. Set to False
+        if the output is going into something like a txt file."""
+        return data_to_string(data, sort_key, wrap_to_terminal)
+
 
 def data_to_string(
     data: list[dict], sort_key: str = None, wrap_to_terminal: bool = True
